@@ -22,8 +22,15 @@
 typedef uint8_t chan;
 typedef uint32_t pel;
 
-/* Yuk. GDKRGB expects data in a specific ordering. */
+/*
+ * Yuk. GDKRGB expects data in a specific ordering:
+ */
 #if defined(DRIFTNET_LITTLE_ENDIAN)
+/*
+ *   - for GTK2 gdk_draw_rgb_32_image():               0x00BBGGRR
+ *   - for GTK3 cairo_image_surface_create_for_data(): 0x00RRGGBB
+ */
+#ifdef DISABLE_GTK3
 #   define PEL(r, g, b)        ((pel)((chan)(r) | ((chan)(g) << 8) | ((chan)(b) << 16)))
 #   define PELA(r, g, b, a)    ((pel)((chan)(r) | ((chan)(g) << 8) | ((chan)(b) << 16) | ((chan)(a) << 24)))
 
@@ -31,7 +38,21 @@ typedef uint32_t pel;
 #   define GETG(p)             ((chan)(((p) & (pel)0x0000ff00) >>  8))
 #   define GETB(p)             ((chan)(((p) & (pel)0x00ff0000) >> 16))
 #   define GETA(p)             ((chan)(((p) & (pel)0xff000000) >> 24))
+#else
+#   define PEL(r, g, b)        ((pel)((chan)(r) << 16 | ((chan)(g) << 8) | ((chan)(b))))
+#   define PELA(r, g, b, a)    ((pel)((chan)(r) << 16 | ((chan)(g) << 8) | ((chan)(b)) | ((chan)(a) << 24)))
+
+#   define GETR(p)             ((chan)(((p) & (pel)0x00ff0000) >> 16))
+#   define GETG(p)             ((chan)(((p) & (pel)0x0000ff00) >>  8))
+#   define GETB(p)             ((chan)(((p) & (pel)0x000000ff)      ))
+#   define GETA(p)             ((chan)(((p) & (pel)0xff000000) >> 24))
+#endif
 #elif defined(DRIFTNET_BIG_ENDIAN)
+/*
+ *  - for GTK2 gdk_draw_rgb_32_image():               0xRRGGBB00
+ *  - for GTK3 cairo_image_surface_create_for_data(): 0xBBGGRR00
+ */
+#ifdef DISABLE_GTK3
 #   define PEL(r, g, b)        ((pel)(((chan)(r) << 24) | ((chan)(g) << 16) | ((chan)(b) << 8)))
 #   define PELA(r, g, b, a)    ((pel)(((chan)(r) << 24) | ((chan)(g) << 16) | ((chan)(b) << 8) | ((chan)(a))))
 
@@ -40,6 +61,14 @@ typedef uint32_t pel;
 #   define GETB(p)             ((chan)(((p) & (pel)0x0000ff00) >>  8))
 #   define GETA(p)             ((chan)(((p) & (pel)0x000000ff)      ))
 #else
+#   define PEL(r, g, b)        ((pel)(((chan)(r) << 8) | ((chan)(g) << 16) | ((chan)(b) << 24)))
+#   define PELA(r, g, b, a)    ((pel)(((chan)(r) << 8) | ((chan)(g) << 16) | ((chan)(b) << 24) | ((chan)(a))))
+
+#   define GETR(p)             ((chan)(((p) & (pel)0x0000ff00) >>  8))
+#   define GETG(p)             ((chan)(((p) & (pel)0x00ff0000) >> 16))
+#   define GETB(p)             ((chan)(((p) & (pel)0xff000000) >> 24))
+#   define GETA(p)             ((chan)(((p) & (pel)0x000000ff)      ))
+#endif
 #   error "no endianness defined"
 #endif
 
