@@ -244,25 +244,25 @@ int validate_options(options_t* options)
      * Check for (at least) one display option (GTK by default), if not in adjunct mode.
      */
     if (!options->adjunct) {
+#if !defined(NO_DISPLAY_WINDOW) && !defined(NO_HTTP_DISPLAY)
         if (options->enable_gtk_display && options->enable_http_display) {
             log_msg(LOG_ERROR, "can't specify -w and -g");
             return FALSE;
         }
 
         if (!(options->enable_gtk_display || options->enable_http_display)) {
-#ifndef NO_DISPLAY_WINDOW
             options->enable_gtk_display = TRUE;
-
-#else
-  #ifndef NO_DISPLAY_WINDOW
-            options->enable_http_display = TRUE;
-  #else
-            log_msg(LOG_WARNING, "this version of driftnet was compiled without any display support");
-            log_msg(LOG_WARNING, "switching to adjunct mode");
-            options->adjunct = TRUE;
-  #endif
-#endif
         }
+#elif !defined(NO_DISPLAY_WINDOW)
+        options->enable_gtk_display = TRUE;
+
+#elif !defined(NO_HTTP_DISPLAY)
+        options->enable_http_display = TRUE;
+#else
+        log_msg(LOG_WARNING, "this version of driftnet was compiled without any display support");
+        log_msg(LOG_WARNING, "switching to adjunct mode");
+        options->adjunct = TRUE;
+#endif
     }
 
     return TRUE;
@@ -325,8 +325,8 @@ void usage(FILE *fp)
 "in its temporary directory. It is assumed that some other process is\n"
 "collecting and deleting the image files.\n"
 "\n"
-"driftnet, copyright (c) 2001-2 Chris Lightfoot <chris@ex-parrot.com>\n"
-"          copyright (c) 2012-18 David Suárez <david.sephirot@gmail.com>\n"
+"driftnet, copyright (c) 2001-2002 Chris Lightfoot <chris@ex-parrot.com>\n"
+"          copyright (c) 2012-2022 David Suárez <david.sephirot@gmail.com>\n"
 "home page: https://github.com/deiv/driftnet\n"
 "old home page: http://www.ex-parrot.com/~chris/driftnet/\n"
 "\n"
